@@ -4,9 +4,9 @@ import shutil
 import argparse 
 import textwrap
 import json
-import parallel_driver
 
 class Open_With_Indent:
+    """This class creates an open file were indentation is tracked"""
 
     def __init__(self,file_name,read_method):
         self.open = open(file_name,read_method)
@@ -34,10 +34,6 @@ class Experiment_Setup:
         if 'years' in dir(self):
             self.years=self.years.split(',')
 
-        #For parallel option:
-        self.parallel_dimension=None
-        self.parallel_dimension_number=0
-
     def prepare_scripts(self):
         """ Prepares the scripts for bash launch """
     
@@ -51,6 +47,7 @@ class Experiment_Setup:
 
         out.writei('#!/bin/bash\n')
         if self.pbs_expt:
+            #If the script is expected to be submitted to a PBS queue, output the required headers:
 	    if self.queue != None: out.writei('#PBS -q {0}\n'.format(self.queue))
             #out.writei('#PBS -l nodes=1:ppn={0},walltime={1}\n'.format(max(self.dim_async,self.m_async),self.walltime))
             out.writei('#PBS -l nodes=1:ppn={0}\n'.format(max(self.dim_async,self.m_async)))
@@ -58,6 +55,7 @@ class Experiment_Setup:
             out.writei('#PBS -o {5}/pbs_out/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
             out.writei('#PBS -e {5}/pbs_err/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
 
+        #Put the header to the diagnostic:
         out.writei('\n')
         out.writei('#The next variable should be empty. If your script completed and some\n')
         out.writei('#years did not process properly (usually due to a timeout on large\n')
@@ -97,17 +95,16 @@ class Experiment_Setup:
         out.writei('CDB_DIAG_HEADER="${CDB_TEMP_DIR}/${CDB_DIAG_HEADER_BASE}"\n')
 
         
-        #LOAD SCRIPT FILE AND CONVERT PARALLEL INSTANCES
+        #Load script file:
         script_file=open(self.diagnostic_dir+'/'+self.diagnostic+'.sh','r')
 
-        parallel_dimension=None
+        #Define instructions:
         instructions={
                       '#!START MONTH LOOP' : start_monthly_loop,
-                      '#!END MONTH LOOP' : end_monthly_loop,
-                      '#!PARALLEL' : parallel_driver.start_parallel_instance,
-                      '#!END PARA' : parallel_driver.end_parallel_instance
+                      '#!END MONTH LOOP' : end_monthly_loop
                     }
 
+        #Loop through lines:
         for line in script_file:
                 out.writei(line)
                 for inst in instructions.keys():
@@ -117,11 +114,13 @@ class Experiment_Setup:
         out.writei('\n')
         out.writei('rm -rf ${CDB_TEMP_DIR}\n')
         out.writei('\n')
+
+        #Finally add BASH code at the end of the script to structure the output like the CMIP5 DRS:
         structure_out_with_cmip5_drs(self,out)
         out.open.close()
 
 def structure_out_with_cmip5_drs(self,out):
-    #Finally, introduce the code to structure the output with the CMIP5 DRS:
+    #code to structure the output with the CMIP5 DRS:
     out.writei('#THE LAST PART OF THIS SCRIPT REORGANIZES THE OUTPUT TO CONFORM WITH THE CMIP5 DRS:\n')
     out.writei('if [ ! -z "$CDB_CMIP5_COMP_LIST" ]; then\n')
     out.inc_indent()
@@ -129,9 +128,8 @@ def structure_out_with_cmip5_drs(self,out):
     out.inc_indent()
     out.writei('FILE=$(echo $FILE_DESC | awk -F\';\' \'{print $1}\')\n')
     out.writei('REALM_FREQ_INFO=$(echo $FILE_DESC | awk -F\';\' \'{print $2}\')\n')
-    out.writei('bash $CDB_SOURCE_DIR/structure_out_as_in.sh -d ${CDB_DIAG_NAME} -m ${CDB_MODEL} -e ${CDB_EXPT} -r ${CDB_RUN_ID} -y ${CDB_YEARS} -f ${REALM_FREQ_INFO} $FILE\n')
-    out.writei('CDB_INFO="This output was created using the Climate Diagnostics Benchmark toolbox,\
-                developed by F. Laliberte and P. J. Kushner, 2012, at the University of Toronto.\
+    out.writei('CDB_INFO="This output was created using the Climate Diagnostics Benchmark toolbox,\n\
+                developed by F. Laliberte and P. J. Kushner, 2012, at the University of Toronto.\n\
                 This code was developed as part of the ExArch project."\n')
     out.writei('\n')
     out.writei('CDB_VAR_LIST=$(cdo showname ${FILE})\n')
@@ -158,6 +156,7 @@ def structure_out_with_cmip5_drs(self,out):
     out.writei('case $CDB_FREQ in\n')
     out.inc_indent()
     out.writei('fx) FILE_NAME="${FILE_NAME}";;\n')
+    out.writei('clim) FILE_NAME="${FILE_NAME}";;\n')
     out.writei('day) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}0101-${CDB_YEAR_END}1231";;\n')
     out.writei('mon) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}01-${CDB_YEAR_END}12";;\n')
     out.writei('yr) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}-${CDB_YEAR_END}";;\n')
@@ -177,6 +176,8 @@ def structure_out_with_cmip5_drs(self,out):
     out.writei('fi\n')
 
 def start_monthly_loop(self,out,line):
+    #This function creates a BASH function within the script. The function processes one month.
+    #Because of the monthly processing this can be parallelized.
     out.writei('function monthly_processing()\n')
     out.writei('{\n')
     out.writei('cat > $3 <<EndOfScriptHeader\n')
@@ -205,6 +206,8 @@ def start_monthly_loop(self,out,line):
     out.writei('\n')
     out.writei('cat >> $3 <<\'EndOfScriptMain\'\n')
     out.writei('\n')
+
+    #Next we generate BASH variables for the easy handling of variables:
     out.writei('#Retrieve path names to data:\n')
     for var_name in self.variable_list.keys():
         if self.variable_list[var_name][0]!='fx':
@@ -218,19 +221,24 @@ def start_monthly_loop(self,out,line):
         out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'=$(cdb_query_archive retrieve '+retrieval_string+')\n')
         out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_TYPE=$(cdb_query_archive retrieve -f '+retrieval_string+')\n')
     out.writei('\n')
+
+    #Next we generate a script line that can be used in CDO:
     out.writei('#Create a cdo retrieval script:\n')
     out.writei('CDO_RETRIEVAL_SCRIPT=""\n')
+    out.writei('if [ -z "$CDB_TIME_OFFSET" ]; then CDB_TIME_OFFSET=0; fi\n')
     for var_name in self.variable_list.keys():
-        var_id = '$CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])
-        out.writei('#Creating retrieval for {0}\n'.format(var_name))
-        out.writei('CDB_VAR=$(echo '+var_id+'| awk -F\'|\' \'{print $1}\')\n')
-        out.writei('CDB_VAR_START=$(echo '+var_id+' | awk -F\'|\' \'{print $2}\')\n')
-        out.writei('CDB_VAR_END=$(echo '+var_id+' | awk -F\'|\' \'{print $3}\')\n')
-        out.writei('CDB_TIME_STEPS=$(echo $(seq $((CDB_VAR_START+1)) $((CDB_VAR_END+1))) | tr \' \' \',\')\n')
-        out.writei('CDO_RETRIEVAL_SCRIPT="${CDO_RETRIEVAL_SCRIPT} -selname,'+var_name+' -seltimestep,${CDB_TIME_STEPS} $CDB_VAR"\n')
+        if not self.variable_list[var_name][0] in ['fx','clim']:
+            var_id = '$CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])
+            out.writei('#Creating retrieval for {0}\n'.format(var_name))
+            out.writei('CDB_VAR=$(echo '+var_id+'| awk -F\'|\' \'{print $1}\')\n')
+            out.writei('CDB_VAR_START=$(echo '+var_id+' | awk -F\'|\' \'{print $2}\')\n')
+            out.writei('CDB_VAR_END=$(echo '+var_id+' | awk -F\'|\' \'{print $3}\')\n')
+            out.writei('CDB_TIME_STEPS=$(echo $(seq $((CDB_VAR_START+1-CDB_TIME_OFFSET)) $((CDB_VAR_END+1+CDB_TIME_OFFSET))) | tr \' \' \',\')\n')
+            out.writei('CDO_RETRIEVAL_SCRIPT="${CDO_RETRIEVAL_SCRIPT} -selname,'+var_name+' -seltimestep,${CDB_TIME_STEPS} $CDB_VAR"\n')
     return self
 
 def end_monthly_loop(self,out,line):
+    #Closing the monthly loop
     out.open.write('EndOfScriptMain\n')
     out.writei('}\n')
 
@@ -240,6 +248,7 @@ def end_monthly_loop(self,out,line):
     out.writei('CDB_YEAR_START=$(echo $CDB_YEARS | awk -F\',\' \'{print $1}\')\n')
     out.writei('CDB_YEAR_END=$(echo $CDB_YEARS | awk -F\',\' \'{print $2}\')\n')
     out.writei('CDB_YEAR=${CDB_YEAR_START}\n')
+
     #Create monthly_scripts
     out.writei('if [ -z "$CDB_YEARS_FIX_LIST" ];then\n')
     out.inc_indent()
@@ -272,13 +281,11 @@ def end_monthly_loop(self,out,line):
 
     #Process the monthly_scripts
     if self.debug:
-        out.writei('for SCRIPT in $(ls ${CDB_TEMP_DIR}/script_????_??.sh); do bash $SCRIPT; rm $SCRIPT; done\n')
-    elif self.m_async == 1:
-        out.writei('for SCRIPT in $(ls ${CDB_TEMP_DIR}/script_????_??.sh); do bash $SCRIPT > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1; rm $SCRIPT; done\n')
+        out.writei('ls ${CDB_TEMP_DIR}/script_????_??.sh | parallel --tmpdir ${CDB_OUT_DIR} --tty "bash {}; rm {}"\n')
     else:
-        out.writei('ls ${CDB_TEMP_DIR}/script_????_??.sh | parallel --tmpdir ${CDB_OUT_DIR} -k -j'+str(self.m_async)+' "bash {}; rm {}" > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1 &\n')
-        out.writei('pid=${!}\n')
-        out.writei('${CDB_SOURCE_DIR}/parallel_ramdisk.sh $pid ${CDB_TEMP_DIR}\n')
+        out.writei('ls ${CDB_TEMP_DIR}/script_????_??.sh | parallel --tmpdir ${CDB_OUT_DIR} -k -j'+str(self.m_async)+' "bash {}; rm {}" > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1 \n')
+        #out.writei('pid=${!}\n')
+        #out.writei('${CDB_SOURCE_DIR}/parallel_ramdisk.sh $pid ${CDB_TEMP_DIR}\n')
         out.writei('rm ${CDB_TEMP_DIR}/script_????_??.sh\n')
     return self
 
@@ -307,7 +314,7 @@ def main():
     setup_group=parser.add_argument_group("Setup","These options must be set when using this script")
     setup_group.add_argument("--debug",dest="debug",
                       default=False, action="store_true",
-                      help="Debug flag. Disables options --m_async and --dim_async.")
+                      help="Debug flag. Disables option --m_async.")
     setup_group.add_argument("--run",dest="run",
                       default=False, action="store_true",
                       help="Launches the scripts.")
@@ -320,6 +327,10 @@ def main():
     proc_group=parser.add_argument_group("Processing",
                             "Use these options to set parallelized options.\n\
                              BEWARE! Asynchronous options are largely untested!")
+    proc_group.add_argument("--m_async",dest="m_async",
+                      default=1,
+                      help="Uses the specified # of processors for the asynchronous processing of months.\n\
+                            be used with --dim_async. Requires NCO version 4.0.0 and above, gnu-parallel.")
     proc_group.add_argument("-P","--pbs_expt",dest="pbs_expt",
                       default=False, action="store_true",
                       help="Prepare a pbs header for each model/run_id/experiment")
@@ -332,25 +343,9 @@ def main():
     proc_group.add_argument("--submit",dest="submit",
                       default=False, action="store_true",
                       help="Submit scripts to the PBS queue")
-    proc_group.add_argument("--m_async",dest="m_async",
-                      default=1,
-                      help="Uses the specified # of processors for the asynchronous processing of months.\n\
-                            Cannot be used with --dim_async. Requires NCO version 4.0.0 and above, gnu-parallel.")
-    proc_group.add_argument("--dim_async",dest="dim_async",
-                      default=1,
-                      help="Uses the specified # of processors in dimension splitting.\n\
-                            Cannot be used with --m_async. Requires NCO version 4.0.0 and above, gnu-parallel.")
 
     options = parser.parse_args()
 
-    #Disable asynchronous computing if debugging
-    if options.debug:
-        options.m_async=1
-        options.dim_async=1
-
-    if options.m_async>1 and options.dim_async>1:
-        raise IOError('Can only specify M_ASYNC>1 or --DIM_ASYNC>1, but not both.')
-
     #Load diagnostic description file:
     if options.diag_header[-3:]=='.gz':
         import gzip
