@@ -12,6 +12,8 @@ def find_path(session,file_expt,path_name,top_name,propagated_values):
     return
 
 def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
+    #Find the intersection of all models that have at least one file in each experiment
+
     diag_tree_desc.append('file_type')
     diag_tree_desc.append('path')
 
@@ -30,7 +32,7 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
 
     for experiment in paths_dict['diagnostic']['experiment_list'].keys():
         for var_name in paths_dict['diagnostic']['variable_list'].keys():
-            if paths_dict['diagnostic']['variable_list'][var_name][0]!='fx':
+            if not paths_dict['diagnostic']['variable_list'][var_name][2][:1] in ['fx','en']:
                 #Do this without fx variables:
                 conditions=[
                              File_Expt.var==var_name,
@@ -49,7 +51,7 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
         #Do it for fx variables:
         model_list_fx=[model[:-1] for model in model_list]
         for var_name in paths_dict['diagnostic']['variable_list'].keys():
-            if paths_dict['diagnostic']['variable_list'][var_name][0]=='fx':
+            if paths_dict['diagnostic']['variable_list'][var_name][2][:1] in ['fx','en']:
                 conditions=[
                              File_Expt.var==var_name,
                              File_Expt.frequency==paths_dict['diagnostic']['variable_list'][var_name][0],
